/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.spzf.ewm.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import lombok.AllArgsConstructor;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.constant.RoleConstant;
import org.springblade.core.tool.utils.AesUtil;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.spzf.ewm.excel.EwmShExcel;
import org.springblade.modules.spzf.ewm.pojo.entity.EwmShEntity;
import org.springblade.modules.spzf.ewm.pojo.vo.EwmShVO;
import org.springblade.modules.spzf.ewm.service.IEwmShService;
import org.springblade.modules.spzf.ewm.wrapper.EwmShWrapper;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * 二维码_商户_终端号 控制器
 *
 * @author wth
 * @since 2024-12-11
 */
@RestController
@AllArgsConstructor
@RequestMapping("/ewmSh")
@Tag(name = "二维码_商户_终端号", description = "二维码_商户_终端号接口")
public class EwmShController extends BladeController {

	private final IEwmShService ewmShService;

	/**
	 * 二维码_商户_终端号 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入ewmSh")
	public R<EwmShVO> detail(EwmShEntity ewmSh) {
		EwmShEntity detail = ewmShService.getOne(Condition.getQueryWrapper(ewmSh));
		detail.setMchtCode(AesUtil.decryptFormBase64ToString(ewmSh.getMchtCode(), "NufN54yGJbxLtPwZ2qH8C6YpN1ZL3KlT"));
		detail.setTerminalId(AesUtil.decryptFormBase64ToString(ewmSh.getTerminalId(), "NufN54yGJbxLtPwZ2qH8C6YpN1ZL3KlT"));
		return R.data(EwmShWrapper.build().entityVO(detail));
	}

	/**
	 * 二维码_商户_终端号 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入ewmSh")
	public R<IPage<EwmShVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> ewmSh, Query query) {
		IPage<EwmShEntity> pages = ewmShService.page(Condition.getPage(query), Condition.getQueryWrapper(ewmSh, EwmShEntity.class));
		return R.data(EwmShWrapper.build().pageVO(pages));
	}


	/**
	 * 二维码_商户_终端号 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入ewmSh")
	public R<IPage<EwmShVO>> page(EwmShVO ewmSh, Query query) {
		IPage<EwmShVO> pages = ewmShService.selectEwmShPage(Condition.getPage(query), ewmSh);
		return R.data(pages);
	}

	/**
	 * 二维码_商户_终端号 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入ewmSh")
	public R save(@Valid @RequestBody EwmShEntity ewmSh) {
		return R.status(ewmShService.save(ewmSh));
	}

	/**
	 * 二维码_商户_终端号 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入ewmSh")
	public R update(@Valid @RequestBody EwmShEntity ewmSh) {
		ewmSh.setMchtCode(AesUtil.encryptToBase64(ewmSh.getMchtCode(), "NufN54yGJbxLtPwZ2qH8C6YpN1ZL3KlT"));
		ewmSh.setTerminalId(AesUtil.encryptToBase64(ewmSh.getTerminalId(), "NufN54yGJbxLtPwZ2qH8C6YpN1ZL3KlT"));
		return R.status(ewmShService.updateById(ewmSh));
	}

	/**
	 * 二维码_商户_终端号 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入ewmSh")
	public R submit(@Valid @RequestBody EwmShEntity ewmSh) {
		return R.status(ewmShService.saveOrUpdate(ewmSh));
	}

	/**
	 * 二维码_商户_终端号 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "逻辑删除", description  = "传入ids")
	public R remove(@Parameter(description = "主键集合", required = true) @RequestParam String ids) {
		return R.status(ewmShService.deleteLogic(Func.toLongList(ids)));
	}

	/**
	 * 导出数据
	 */
	@PreAuth(RoleConstant.HAS_ROLE_ADMIN)
	@GetMapping("/export-ewmSh")
	@ApiOperationSupport(order = 8)
	@Operation(summary = "导出数据", description  = "传入ewmSh")
	public void exportEwmSh(@Parameter(hidden = true) @RequestParam Map<String, Object> ewmSh, BladeUser bladeUser, HttpServletResponse response) {
		QueryWrapper<EwmShEntity> queryWrapper = Condition.getQueryWrapper(ewmSh, EwmShEntity.class);
		//if (!AuthUtil.isAdministrator()) {
		//	queryWrapper.lambda().eq(EwmShEntity::getTenantId, bladeUser.getTenantId());
		//}
		//queryWrapper.lambda().eq(EwmShEntity::getIsDeleted, BladeConstant.DB_NOT_DELETED);
		List<EwmShExcel> list = ewmShService.exportEwmSh(queryWrapper);
		ExcelUtil.export(response, "二维码_商户_终端号数据" + DateUtil.time(), "二维码_商户_终端号数据表", list, EwmShExcel.class);
	}

}
